<?php
namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\OpensslCrypt;
use OrganizationsManagement\Factory;
use Teknisa\Model\Teknisa\Entities\OmProduto;
use Teknisa\Model\Teknisa\Entities;
use Teknisa\Model\Services\Entities as ServicesEntities;
use Teknisa\Model\Teknisa\Repositories;
use Teknisa\Libs\Util\Environment as TeknisaEnvironment;
use OrganizationsManagement\Util\InstanceProvider;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Model\Teknisa\Entities\OmConfigMedidaVariavel;
use OrganizationsManagement\Util\Messages;
use \Zeedhi\Framework\Util\Functions;

class Produto
{
    const VARIABLE_MEASURE_COLLECTION = 'VariableMeasure';
    const GENERAL_COLLECTION = 'GeneralCollection';
    const PRODUCT_KEY = 'product';

    /** @var Repositories\OmProduto **/
    protected $repositoryProduto;

    /** @var Repositories\OmProdutoOrg **/
    protected $repositoryProdutoOrg;
    
    /** @var Repositories\OmProdutoElemento **/
    protected $repositoryProdutoElemento;
    
    /** @var Repositories\OmProdutoOrgElemento **/
    protected $repositoryProdutoOrgElemento;

    /** @var Utilities **/
    protected $utilitarios;

    /** @var EntityManager **/
    protected $entityManager;

    /** @var TeknisaEnvironment **/
    protected $environment;

    /** @var Factory\Produto **/
    protected $factoryProduto;
    
    /** @var Factory\ProdutoOrg **/
    protected $factoryProdutoOrg;

    public function __construct(
        EntityManager $entityManager, Utilities $utilitarios, TeknisaEnvironment $environment, 
        Factory\Produto $factoryProduto, Factory\ProdutoOrg $factoryProdutoOrg
    ){
        $this->utilitarios = $utilitarios;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->factoryProduto = $factoryProduto;
        $this->factoryProdutoOrg = $factoryProdutoOrg;
        $this->repositoryProduto = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProduto');
        $this->repositoryProdutoOrg = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoOrg');
        $this->repositoryProdutoElemento = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoElemento');
        $this->repositoryProdutoOrgElemento = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoOrgElemento');
    }

    public function save($id, $dsurlJsonMenu, $idStatus, $dschavenotif, $cdappnotif, $idprodutosaas, $idpercriarchamadas, $idperdeploy, $idProdutoAssistant, $idUtAssMultVer){
        $produto = $this->repositoryProduto->findOneBy(array('id' => $id));

        $produto->setDsurlJsonMenu($dsurlJsonMenu);
        $produto->setIdStatus($idStatus);
        $produto->setDschavenotif($dschavenotif);
        $produto->setCdappnotif($cdappnotif);
        $produto->setIdprodutosaas($idprodutosaas);
        $produto->setCdoperultatu($this->environment->getCurrentUserId());
        $produto->setNrorgultatu($this->environment->getCurrentOrganizationId());
        $produto->setIdpercriarchamado($idpercriarchamadas);
        $produto->setIdperdeploy($idperdeploy);
        $produto->setIdprodutoassistant($idProdutoAssistant);
        $produto->setIdutassmultver($idUtAssMultVer);

        $this->entityManager->persist($produto);
        $this->entityManager->flush();
    }

    public function saveOrganizationProducts($produtosOrg) {
        $conexao = $this->entityManager->getConnection();
        $conexao->beginTransaction();
        $dateNow = new \DateTime();
        $currentUserId = $this->environment->getCurrentUserId();
        $currentOrganizationId = $this->environment->getCurrentOrganizationId();
        $organizacaoRepository = InstanceProvider::getRepository()->getOrganizacao();
        
        try {
            
            foreach ($produtosOrg as $produtoOrg) {
                if (!empty($produtoOrg['ID'])){
                    $idAtivo            = !empty($prdutoOrg['IDATIVO']) ? $produtoOrg['IDATIVO'] : OmProduto::ID_ACTIVE;
                    $omProdutoOrgEntity = $this->repositoryProdutoOrg->findOneBy(array('nrprodutoId' => $produtoOrg['ID'], 'nrorgId'=> $produtoOrg['NRORG_ID']));
                    $idProdSaas         = $produtoOrg['IDPRODUTOSAAS'] === OmProduto::HERDA_ORGANIZACAO ? $organizacaoRepository->findOneBy(array('nrorg' => $produtoOrg['NRORG_ID']))->getIdorgsaas() : $produtoOrg['IDPRODUTOSAAS'];
                    if($idProdSaas != "S"){
                        $idProdSaas = "N";
                    }
                    $omProdutoOrg       = $this->factoryProdutoOrg->save($omProdutoOrgEntity, $produtoOrg['NRORG_ID'], $produtoOrg['ID'],
                                          $currentUserId, $currentOrganizationId, $dateNow, $idAtivo, $idProdSaas);
                    $this->entityManager->persist($omProdutoOrg);
                } else {
                    throw new \Exception("É necessário id para cadastrar");
                }
            }
            $this->entityManager->flush();
            $conexao->commit();
        } catch (\Exception $ex) {
            $conexao->rollback();
            throw $ex;
        }
    }

    public function removeProdutoPorOrganizacao($id) {
        $conexao = $this->entityManager->getConnection();
        $conexao->beginTransaction();
        try {
            $this->removeProductsAndYoursElementsByOrganizationProductId($id);
            $conexao->commit();
        } catch (\Exception $ex) {
            $conexao->rollback();
            throw $ex;
        }
    }
    
    public function removeProductsAndYoursElementsByOrganizationProductId($id){
        $omProdutoOrgEntity = $this->repositoryProdutoOrg->findOneBy(array('id' => $id));
        $orgElements = $this->repositoryProdutoOrgElemento->findBy(array('nrprodutoOrgId' => $id));
        $orgVersion =InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findBy(array('nrprodutoOrgId' => $id));
        foreach ($orgElements as $element) {
            $elementoContrato =InstanceProvider::getRepository()->getOmProdutoOrgElementoContrato()->findBy(array('nrprodutoOrgElementoId' => $element->getId()));
            foreach($elementoContrato as $contrato){
                $this->entityManager->remove($contrato);
            }
            $this->entityManager->remove($element);
        }
        foreach ($orgVersion as $versions) {
            $this->entityManager->remove($versions);
        }
        $this->entityManager->flush();
        $this->entityManager->remove($omProdutoOrgEntity);
        $this->entityManager->flush();
    }

    public function obtemTodos($filters = array(), $page = 0, $pageSize = 0) {
        $conditions = Utilities::createQueryFiltered($filters);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return $this->repositoryProduto->obtemTodos($conditions, $limit, $offset);
    }

    public function getProductRelationship($filters = array(), $page = 0, $pageSize = 0) {
        $productRepository = InstanceProvider::getRepository()->getProduto();
        $conditions = Utilities::createQueryFiltered($filters);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $products = $productRepository->getProductRelationship($conditions, $limit, $offset);
        foreach($products as &$product){
            if($product["TYPE_ID"] == "I"){
                $product["TYPE_LABEL"] = "Produto (Zeedhi: App, Browser, Mobile)";
                $product["PRODUCT_RELATIONSHIP"] = "PM: ".$product["PRODUCT_NAME_I"];
            }else if($product["TYPE_ID"] == "S"){
                $product["TYPE_LABEL"] = "Produto (Delphi: Windows)";
                $product["PRODUCT_RELATIONSHIP"] = "GEP: ".$product["PRODUCT_TYPE_NAME"]." - ".$product["PRODUCT_NAME_S"];
            }else{
                $product["TYPE_LABEL"] = "Nenhum";
                $product["PRODUCT_RELATIONSHIP"] = null;
            }
        }
        return $products;
    }

    public function findProductCallProducts($filters = array(), $page = 0, $pageSize = 0) {
        $productRepository = InstanceProvider::getRepository()->getProduto();
        $conditions = Utilities::createQueryFiltered($filters);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $orgId = $this->environment->getCurrentOrganizationId();
        return $productRepository->findProductCallProducts($orgId, 0, $conditions, $limit, $offset);
    }

    public function findProductCallProductsFromRichClient($filters = array(), $page = 0, $pageSize = 0, $productType) {
        $conditions = Utilities::createQueryFiltered($filters);
        $productRepository = InstanceProvider::getRepository()->getProduto();
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return $productRepository->findProductCallProductsFromRichClient($productType, '', '', '', $conditions);
    }

    public function findProductCallProductTypes($filter){
        $productRepository = InstanceProvider::getRepository()->getProduto();
        return $productRepository->findProductCallProductTypes(Utilities::createQueryFiltered($filter));
    }
    
    public function findProduct($productId){
        return $this->repositoryProduto->getProduct($productId);
    }
    
    public function obtemProdutosOrganizacao($organizationId, $filters){
        $conditions = Utilities::createQueryFiltered($filters);
        $products = $this->repositoryProdutoOrg->obtemTodos($organizationId, $conditions);
        return $products;
    }

    public function createNewProductOrg($nrorg, $filters, $nrProduto){
        $new = $this->factoryProdutoOrg->save(null, $nrorg, $nrProduto,
            $this->environment->getCurrentUserId(), $this->environment->getCurrentOrganizationId(), new \DateTime());
        $this->entityManager->persist($new);
        $this->entityManager->flush();
        return $this->obtemProdutosOrganizacao($nrorg, $filters);
    }

    public function obtemProdutosPorOrganizacaoNaoIncluidos($organizationId, $filters){
        $conditions = Utilities::createQueryFiltered($filters);
        return $this->repositoryProdutoOrg->obtemTodosNaoIncluidos($organizationId, $conditions);
    }

    public function obtemProdutosDoDevOps(){
        $token = $this->obtemTokenPeloDevOps();
        
        if(empty($token)){
            throw new \Exception("O token para comunicação com o DevOps é inválido.");
        }

        $data = array(
            "requestType" => "FilterData",
            "row" => array()
        );

        return $this->utilitarios->chamaWebService(
            $token, '/product', $data, 'product', 
            Utilities::SECRET_CLIENTE_DEVOPS, $this->utilitarios->getDevOpsURL()
        );
    }

    public function obtemTokenPeloDevOps(){
        $token = $this->utilitarios->obtemTokenDevOpsDaSessao();

        if(empty($token)){
            $idCliente = Utilities::ID_CLIENTE_LIBRARIES;
            $secretCliente = Utilities::SECRET_CLIENTE_LIBRARIES;
            $param = array(
                'CLIENT_ID' => $idCliente,
                'CLIENT_SECRET' => $secretCliente
            );
            $token = $this->utilitarios->chamaWebService(
                null, '/lib_requestToken', $param, 'token',
                $secretCliente, $this->utilitarios->getDevOpsURL()
            );
            $token = empty($token) ? null : $token[0];
            $this->utilitarios->defineTokenDevOpsNaSessao($token);
        }

        return $token;
    }

    /**
     * Recebe um array comum onde cada ítem é um array associativo. Cria então um
     * novo array, dessa vez associativo, onde cada chave é a chave associativa
     * informada e, cada ítem dessa chave, o ítem dono daquele valor.
     *
     * Exemplo:
     * ["5"] => ["ID" => 5, "NMPRODUTO" => "Produto #5"]
     * ["3"] => ["ID" => 3, "NMPRODUTO" => "Produto #3"]
     **/
    private function mapeiaProdutosComArrayAssociativo($produtos, $chaveAssociativa){
        $produtosMapeado = array();
        foreach($produtos as $produto){
            $chave = $produto[$chaveAssociativa];
            $produtosMapeado[$chave] = $produto;
        }
        return $produtosMapeado;
    }

    public function synchronize(){
        $this->entityManager->clear();
        $conexao = $this->entityManager->getConnection();
        $conexao->beginTransaction();

        try {
            $maiorIdProduto = $this->repositoryProduto->obtemMaiorId();

            $produtosDevOpsDefault = $this->obtemProdutosDoDevOps();
            $produtosDevOpsIds = array_map(function ($produto) {
                return $produto["ID"];
            }, $produtosDevOpsDefault);

            $produtosDevOps = $this->repositoryProduto->filterDevOpsProducts($produtosDevOpsIds);
            $produtosOrganizations = $this->obtemTodos();

            // Primeiro, mapeamos todos os produtos do OM em um array associativo,
            $produtosOrganizationsMapeado = $this->mapeiaProdutosComArrayAssociativo($produtosOrganizations, "NREXTERNAL_PRODUCT_ID");

            // Agora, navegamos entre os produtos do DevOps para checar
            // quais produtos dele estão gravados no OM ($produtosOrganizationsMapeado).
            $produtosDevOpsMapeado = array();
            foreach ($produtosDevOps as $produto) {
                $idProduto = $produto["ID"];

                // Aproveitamos e já criamos um array mapeado para o $produtosDevOps,
                // evitando um outro foreach para isso.
                $produtosDevOpsMapeado[$idProduto] = $produto;

                // Se o OM não tem esse produto
                if (empty($produtosOrganizationsMapeado[$idProduto])) {
                    $id = $maiorIdProduto;
                    $nome = $produto["NAME"];
                    $nrexternalorg = $produto["ORGANIZATION_ID"];
                    $nrexternalProductId = $produto["ID"];
                    $idstatus = $produto["IS_ACTIVE"] && $produto["IS_ACTIVE"] == 'Y' ? 'A' : 'I';

                    // Criamos o produto
                    $produtoCriado = $this->factoryProduto->cria($id, $nome, $nrexternalorg, $nrexternalProductId, $idstatus);
                    $this->entityManager->persist($produtoCriado);
                } // Se o OM tem o produto
                else {
                    $produtoParaModificar = $this->repositoryProduto->findOneBy(array('nrexternalProductId' => $idProduto));
                    $mapaDeConversaoDeNomes = array(
                        "ID" => "NREXTERNAL_PRODUCT_ID",
                        "NAME" => "NMPRODUTO",
                        "IS_ACTIVE" => "IDSTATUS",
                        "ORGANIZATION_ID" => "NREXTERNAL_ORG"
                    );

                    // Para cada propriedade (equivalente às colunas da tabela)
                    foreach ($produto as $propriedade => $valor) {
                        $propriedade = Functions::arrayKeyExists($propriedade, $mapaDeConversaoDeNomes) ? $mapaDeConversaoDeNomes[$propriedade] : $propriedade;

                        // Se o valor da propriedade no DevOps for diferente do OM, atualizamos o valor no OM
                        if (Functions::arrayKeyExists($propriedade, $produtosOrganizationsMapeado[$idProduto]) &&
                            $valor != $produtosOrganizationsMapeado[$idProduto][$propriedade]
                        ) {
                            if ($propriedade == "IDSTATUS") {
                                $valor = $valor == 'Y' ? 'A' : 'I';
                            }

                            $metodoSetter = $this->utilitarios->criaMetodoSetterPorString($propriedade);
                            $produtoParaModificar->$metodoSetter($valor);
                        }
                    }
                    $this->entityManager->persist($produtoParaModificar);
                }

                $maiorIdProduto++;
            }

            // Agora, navegamos entre os produtos do OM
            foreach ($produtosOrganizations as $produto) {
                $idProduto = $produto["NREXTERNAL_PRODUCT_ID"];

                // Se o DevOps não tem esse produto, removemos do OM
                if (empty($produtosDevOpsMapeado[$idProduto])) {
                    /** @var OmProduto $produtoParaRemover */
                    $produtoParaRemover = $this->repositoryProduto->findOneBy(array(
                        'nrexternalProductId' => $idProduto)
                    );
                    $this->deleteProductRelations($produtoParaRemover);
                    $this->entityManager->flush();
                    $this->entityManager->remove($produtoParaRemover);
                }
            }
            $this->entityManager->flush();
            $conexao->commit();
        } catch (\Exception $ex) {
            $conexao->rollback();
            throw $ex;
        }
    }
    
    public function saveProductAppCode($dschavenotif, $cdappnotif, $productId, $idProdSaas)
    {
        $connection = $this->entityManager->getConnection();
        $productOrgRepository = InstanceProvider::getRepository()->getOmProduto();
        $productEntity = null;
        try {
            $connection->beginTransaction();
            $productEntity = $productOrgRepository->findOneBy(array('id' => $productId));
            $productEntity->setCdappnotif($cdappnotif);
            $productEntity->setDschavenotif($dschavenotif);
            $productEntity->setIdprodutosaas($idProdSaas);
            
            $this->entityManager->persist($productEntity);
            $this->entityManager->flush();
            $connection->commit();
            return array('ID' => $productEntity->getId());
        } catch (Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function findProductRepositories($filters, $nrProdutoId) {
        $conditions = Utilities::createQueryFiltered($filters);
        return InstanceProvider::getRepository()->getOmProdutoRepositorio()->findProductRepositories($conditions, $nrProdutoId);
    }

    public function findNotInsertedProductRepositories($filters, $externalProductId, $nrProdutoId) {
        $conditions = Utilities::createQueryFiltered($filters);
        return InstanceProvider::getRepository()->getOmProdutoRepositorio()->
            findNotInsertedProductRepositories($conditions, $externalProductId, $nrProdutoId);
    }

    public function saveProductRelationship($cdProduto,$projectId = null,$cdProdcomp = null,$cdProdgrup = null){
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $productRepository = InstanceProvider::getRepository()->getProduto();
            $productEntity = $productRepository->findOneBy(array('cdproduto' => $cdProduto));
            $productEntity = InstanceProvider::getFactory()->getProduto()
                ->updateProduto($productEntity,$projectId, $cdProdcomp, $cdProdgrup);
            $this->entityManager->persist($productEntity);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function saveProductRepository($id, $nrProdutoId, $nrExternalRepositoryId, $dsdirrelmigdb, $idAtivo) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $productRepository = InstanceProvider::getRepository()->getOmProdutoRepositorio();

            /** @var Entities\OmProdutoRepositorio $productEntity */
            $productEntity = $productRepository->findOneBy(array('id' => $id));

            $productEntity = InstanceProvider::getFactory()->getProdutoRepositorio()
                ->save($productEntity, $nrProdutoId, $nrExternalRepositoryId, $dsdirrelmigdb, $idAtivo);

            $this->entityManager->persist($productEntity);
            $this->entityManager->flush();
            $connection->commit();
            return array('ID' => $productEntity->getId());
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function deleteProductRepository($id) {
        $productRepository = InstanceProvider::getRepository()->getOmProdutoRepositorio();

        /** @var Entities\OmProdutoRepositorio $productEntity */
        $productEntity = $productRepository->findOneBy(array('id' => $id));

        $this->entityManager->remove($productEntity);
        $this->entityManager->flush();
    }

    public function findVariableMeasurementSettings($filters, $nrProdutoId) {
        $conditions = Utilities::createQueryFiltered($filters);
        return InstanceProvider::getRepository()->getOmConfigMedidaVariavel()->findVariableMeasurementSettings($conditions, $nrProdutoId);
    }

    public function saveVariableMeasurementSettings($id, $nrProdutoId, $phpScriptLog, $phpScriptCount, $qtdDiasAviso, 
                                                    $qtdDiasReaplicar, $cdTpMedidaServ, $idAtivo, $nrprodutoIdRel, $idTpScript) {
        $this->validateVariableMeasurementSettingsSave($id, $nrProdutoId, $cdTpMedidaServ);
        $omProduto = InstanceProvider::getRepository()->getOmProduto()->find($nrprodutoIdRel);
        /** @var Entities\OmConfigMedidaVariavel $configMedidaVariavelEntity */
        $configMedidaVariavelEntity = InstanceProvider::getRepository()->getOmConfigMedidaVariavel()->findOneBy(array('id' => $id));
        $configMedidaVariavelEntity = InstanceProvider::getFactory()->getConfigMedidaVariavel()->save($configMedidaVariavelEntity,
            $nrProdutoId, $phpScriptLog, $phpScriptCount, $qtdDiasAviso, $qtdDiasReaplicar, $cdTpMedidaServ, $idAtivo, $omProduto, $idTpScript);
        $this->entityManager->persist($configMedidaVariavelEntity);
        $this->entityManager->flush();
        return array('ID' => $configMedidaVariavelEntity->getId());
    }
    
    public function validateVariableMeasurementSettingsSave($id, $nrProdutoId, $cdTpMedidaServ) {
        $variableMeasure = InstanceProvider::getRepository()->getFctTipomedidaserv()->findOneBy(
            array('cdtpmedidaserv' => $cdTpMedidaServ, 'nrorg' => $this->environment->getCurrentOrganizationId())
        );
        if($variableMeasure->getIdtpcoletamedidaserv() == 'TEKNISA') {
            $configMedidaVariavel = InstanceProvider::getRepository()->getOmConfigMedidaVariavel()->findOneBy (
                array('nrprodutoId' => $nrProdutoId, 'cdtpmedidaserv' => $cdTpMedidaServ)
            );
            if($configMedidaVariavel && $configMedidaVariavel->getId() != $id) {
                throw new \Exception(Messages::ERR_EXISTS_OTHER_TEKNISA_VARIABLE_MEASURE);
            }
        }
    }

    public function deleteVariableMeasurementSettings($id) {
        /** @var Entities\OmConfigMedidaVariavel $configMedidaVariavelEntity */
        $configMedidaVariavelEntity = InstanceProvider::getRepository()->getOmConfigMedidaVariavel()->findOneBy(array('id' => $id));
        $this->entityManager->remove($configMedidaVariavelEntity);
        $this->entityManager->flush();
    }

    public function findTipoMedidaServ($filters, $page, $pageSize) {
        $conditions = Utilities::createQueryFiltered($filters);
        $limit      = $page * $pageSize;
        $offset     = ($page - 1) * $pageSize;
        $nrOrg      = $this->environment->getCurrentOrganizationId();
        return InstanceProvider::getRepository()->getOmConfigMedidaVariavel()->findTipoMedidaServ($conditions, $nrOrg, $limit, $offset);
    }

    public function saveTipoMedidaServ($cdTpMedidaServ, $idTpColetaMedidaServ) {
        /** @var ServicesEntities\FctTipomedidaserv $variableMeasure */
        $variableMeasure = InstanceProvider::getRepository()->getFctTipomedidaserv()->findOneBy(
            array('cdtpmedidaserv' => $cdTpMedidaServ, 'nrorg' => $this->environment->getCurrentOrganizationId())
        );
        $variableMeasure->setIdtpcoletamedidaserv($idTpColetaMedidaServ);
        $this->entityManager->flush();
        return array($cdTpMedidaServ);
    }

    public function findCollectionVariableMeasures($conditions, $dataSourceName, $page, $pageSize, $orderBy, $nrProdutoId, $nrOrgId) {
        $conditionsFixed = $this->fixConditionsToZmartFilterCriteria($conditions);
        $conditions = $conditionsFixed['CONDITIONS'];

        $zmartFilterCriteria = $this->utilitarios->convertFilterCriteriaToZmartFilterCriteria($conditions, $dataSourceName);
        if($nrProdutoId == 0) {
            $nrProdutoId = intval($nrProdutoId);
            $zmartFilterCriteria->addCondition('productId', '=', strval($nrProdutoId));
        } else {
            /** @var Entities\OmProduto $produto */
            $produto = InstanceProvider::getRepository()->getOmProduto()->find($nrProdutoId);
            $zmartFilterCriteria->addCondition('productId', '=', strval($produto->getNrexternalProductId()));
        }

        $zmartFilterCriteria->setPage($page);
        $zmartFilterCriteria->setPageSize($pageSize);
        if(empty($orderBy)) {
            $zmartFilterCriteria->addOrderBy('date', FilterCriteria::ORDER_DESC);
        } else {
            foreach ($orderBy as $idx => $order) {
                $zmartFilterCriteria->addOrderBy($idx, $order);
            }
        }

        $criteria = InstanceProvider::getMongoQueryBuilder()->buildQueryFromFilterCriteria($zmartFilterCriteria);
        $this->setMappedLikeAllCriteria($criteria, $conditionsFixed['MAPPED_LIKE_ALL']);
        $registers = InstanceProvider::getLicenseMongoDB()->aggregate($this->getVariableMeasureCollection($nrOrgId), $criteria);
        foreach($registers as &$register) {
            $register['date']           = \DateTime::createFromFormat('Ymd', $register['date'])          ->format('d/m/Y');
            $register['endCompetence']  = \DateTime::createFromFormat('Ymd', $register['endCompetence']) ->format('d/m/Y');
            $register['initCompetence'] = \DateTime::createFromFormat('Ymd', $register['initCompetence'])->format('d/m/Y');
            $register['serviceName']    = isset($register['serviceName']) ? $register['serviceName'] : null;
            $register['serviceCode']    = isset($register['serviceCode']) ? $register['serviceCode'] : null;
        }
        return $registers;
    }

    public function findCollectionGeneral($conditions, $dataSourceName, $page, $pageSize, $orderBy, $nrProdutoId, $nrOrgId) {
        $conditionsFixed = $this->fixConditionsToZmartFilterCriteria($conditions);
        $conditions = $conditionsFixed['CONDITIONS'];

        /** @var Entities\OmProduto $produto */
        $produto = InstanceProvider::getRepository()->getOmProduto()->find($nrProdutoId);
        $zmartFilterCriteria = $this->utilitarios->convertFilterCriteriaToZmartFilterCriteria($conditions, $dataSourceName);
        $zmartFilterCriteria->addCondition('productId', '=', strval($produto->getNrexternalProductId()));
        $zmartFilterCriteria->setPage($page);
        $zmartFilterCriteria->setPageSize($pageSize);
        if(empty($orderBy)) {
            $zmartFilterCriteria->addOrderBy('date', FilterCriteria::ORDER_DESC);
        } else {
            foreach ($orderBy as $idx => $order) {
                $zmartFilterCriteria->addOrderBy($idx, $order);
            }
        }
        $criteria = InstanceProvider::getMongoQueryBuilder()->buildQueryFromFilterCriteria($zmartFilterCriteria);
        $this->setMappedLikeAllCriteria($criteria, $conditionsFixed['MAPPED_LIKE_ALL']);
        $registers = InstanceProvider::getLicenseMongoDB()->aggregate($this->getGeneralCollection($nrOrgId), $criteria);
        foreach($registers as &$register) {
            $register['date']     = \DateTime::createFromFormat('Ymd', $register['date'])    ->format('d/m/Y');
            $register['dtLimite'] = \DateTime::createFromFormat('Ymd', $register['dtLimite'])->format('d/m/Y');
        }
        return $registers;
    }

    private function getVariableMeasureCollection($organizationId) {
        return self::VARIABLE_MEASURE_COLLECTION . '_' . $organizationId;
    }

    private function getGeneralCollection($organizationId)
    {
        return self::GENERAL_COLLECTION . '_' . $organizationId;
    }

    private function fixConditionsToZmartFilterCriteria($conditions) {
        $mappedLikeAll = null;
        foreach($conditions as $idx => &$condition) {
            $name = isset($condition['name']) ? $condition['name'] : $condition['columnName'];
            if($condition['operator'] == 'LIKE_I' || $condition['operator'] == 'LIKE') {
                $condition['operator'] = 'LIKE_ALL';
            }
            if($name == 'date') {
                $condition['value'] = array(
                    intval(\DateTime::createFromFormat('d/m/Y', $condition['value'][0])->format('Ymd')),
                    intval(\DateTime::createFromFormat('d/m/Y', $condition['value'][1])->format('Ymd'))
                );
            } else if($name == 'initCompetence' || $name == 'endCompetence'){
                $condition['value'] = intval(\DateTime::createFromFormat('d/m/Y', $condition['value'])->format('Ymd'));
            }
            if ($condition['operator'] == 'MAPPED_LIKE_ALL') {
                $mappedLikeAll = $condition;
                unset($conditions[$idx]);
            }
        }
        return array('CONDITIONS' => $conditions, 'MAPPED_LIKE_ALL' => $mappedLikeAll);
    }

    private function setMappedLikeAllCriteria(&$criteria, $mappedLikeAll) {
        if(!empty($mappedLikeAll)) {
            if (!isset($criteria[0]['$match'])) {
                $criteria['$match'] = array();
            }
            $name = isset($mappedLikeAll['name']) ? $mappedLikeAll['name'] : $mappedLikeAll['columnName'];
            $mappedLikeAllColumns = array('measureName', 'contractId', 'count', 'operatorName', 'serviceName');
            $value = str_replace('%', '', $mappedLikeAll['value'][0]['_ALL']);
            foreach ($mappedLikeAllColumns as $column) {
                $criteria[0]['$match']['$or'][] = array($column => array('$regex' => ".*{$value}.*"));
            }
        }
    }

    public function  findProductAvailableVersions($devopsId, $nrorgId, $nrProduto, $filters) {
        $conditions = Utilities::createQueryFiltered($filters);
        $omProduto = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findOmProdutoByDevopsProductId($devopsId);
        $productOrg = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findProductOrgByDevopsProductId($omProduto[0]['ID'], $nrorgId);
        $moduleFilter = null;
        if(!$productOrg){
            $productOrgObj['ID'] = $omProduto[0]['ID'];
            $productOrgObj['IDPRODUTOSAAS'] = $omProduto[0]['IDPRODUTOSAAS'];
            $productOrgObj['NRORG_ID'] = $nrorgId;
            $this->saveOrganizationProducts([$productOrgObj]);
            $productOrg = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findProductOrgByDevopsProductId($omProduto[0]['ID'], $nrorgId);
    
        }
    
        if($nrProduto != $omProduto[0]['ID']) {
            $parentProductOrg = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findProductOrgByDevopsProductId( $nrProduto, $nrorgId);
            $moduleFilter = $parentProductOrg[0]['ID'];
        }
        return InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findProductAvailableVersions($productOrg[0]['ID'], $moduleFilter, $conditions);
    }


    public function findProductAvailableLatestVersions($nrProdutoOrgId, $filters) {
        $productVersion = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findVersionModulesConfigurationByProdutoOrg($nrProdutoOrgId);
        if(empty($productVersion)){
            $devOpsId = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findDevopsIdByProductOrg($nrProdutoOrgId);
            $obj = '{
                "product": {
                    "baseUrl": "",
                    "devOpsId":' . $devOpsId['NREXTERNAL_PRODUCT_ID'] .'
                }
            }';
            $modulesConfig = array(
                "MODULES_CONFIGURATION" => $obj,
            );
        } else {
            $modulesConfig = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findVersionModulesConfiguration(null, $productVersion['ID']);        
        }
        $modulesConfiguration = array();
        $productDevOpsId = null;
        if(!empty($modulesConfig)) {
            $modulesConfig = @json_decode($modulesConfig['MODULES_CONFIGURATION'], true);
            if(empty($modulesConfig)) {
                return $modulesConfiguration;
            }
            foreach($modulesConfig as $idx => $config) {
                
                if(!isset($config['onlyControl']) || !$config['onlyControl']) {
                    $product = $this->repositoryProduto->findOneBy(array('nrexternalProductId' => $config['devOpsId']));
                    $isProduct = $idx == self::PRODUCT_KEY;
                    if ($isProduct) {
                        $productDevOpsId = $config['devOpsId'];
                    }
                    $modulesConfiguration[] = array (
                        'ID'          => $config['devOpsId'],
                        'MAX_VERSION' => $isProduct ? null : $config['maxVersion'],
                        'PARENT'      => $isProduct ? null : $productDevOpsId,
                        'NAME'        => $product ? $product->getNmproduto() : ''
                    );
                }
            }
        }
        return $modulesConfiguration;
    }

    public function findLTSVersions($filters, $nrProdutoId, $page, $pageSize, $notInsertedOnly, $productOrgId = null) {
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $conditions = Utilities::createQueryFiltered($filters);
        return InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findLTSVersions($conditions, $nrProdutoId, $limit, $offset, $notInsertedOnly, $productOrgId);
    }

    public function findLTSVersionsByDevopsId($filters, $devopsId, $page, $pageSize, $notInsertedOnly, $productOrgId = null) {
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $conditions = Utilities::createQueryFiltered($filters);
        return InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findLTSVersionsByDevopsId($conditions, $devopsId, $limit, $offset, $notInsertedOnly, $productOrgId);
    }

    public function saveLTSVersionsByDevopsId($rows) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $dateNow = new \DateTime();
            $currentUserId = $this->environment->getCurrentUserId();
            $currentOrganizationId = $this->environment->getCurrentOrganizationId();
            $parentId = null;
            foreach($rows as $row) {
                $produtoOrgVersao = null;
                if(isset($row['ID'])) {
                    /** @var Entities\OmProdutoOrgVersao $produtoOrgVersao */
                    $produtoOrgVersao = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->find($row['ID']);
                }
                $omProduto = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findOmProdutoByDevopsProductId($row['DEVOPS_ID']);
                $productOrg = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findProductOrgByDevopsProductId($omProduto[0]['ID'],  $row['NRORG_ID']);
                $idAtivo = isset($row['IDATIVO']) ? $row['IDATIVO'] : 'S';
                if($row['PRODUCT_ID'] != $omProduto[0]['ID']){
                    $productOrgParent = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findProductOrgByDevopsProductId($row['PRODUCT_ID'],  $row['NRORG_ID']);
                    $parentId = $productOrgParent[0]['ID'];
                }
                $produtoOrgVersao = InstanceProvider::getFactory()->getProdutoOrgVersao()->save($produtoOrgVersao,
                $productOrg[0]['ID'], $row['PRODUCT_VERSION_ID'], $currentUserId, $currentOrganizationId, $dateNow, $idAtivo, $parentId);
            

                $this->entityManager->persist($produtoOrgVersao);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function saveLTSVersions($rows) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $dateNow = new \DateTime();
            $currentUserId = $this->environment->getCurrentUserId();
            $currentOrganizationId = $this->environment->getCurrentOrganizationId();

            foreach($rows as $row) {
                $produtoOrgVersao = null;
                if(isset($row['ID'])) {
                    /** @var Entities\OmProdutoOrgVersao $produtoOrgVersao */
                    $produtoOrgVersao = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->find($row['ID']);
                }
                $idAtivo = isset($row['IDATIVO']) ? $row['IDATIVO'] : 'S';
                $produtoOrgVersao = InstanceProvider::getFactory()->getProdutoOrgVersao()->save($produtoOrgVersao,
                    $row['NRPRODUTO_ORG_ID'], $row['PRODUCT_VERSION_ID'], $currentUserId, $currentOrganizationId, $dateNow, $idAtivo);

                $this->entityManager->persist($produtoOrgVersao);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function deleteProductAvailableVersions($rows) {
        foreach($rows as $row) {
            $id = $row['ID'];
            /** @var Entities\OmProdutoOrgVersao $produtoOrgVersao */
            $produtoOrgVersao = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->find($id);
            $this->entityManager->remove($produtoOrgVersao);
            $this->entityManager->flush();
        }
        
    }

    public function findVersionModulesConfiguration($filters, $productVersionId) {
        $conditions    = Utilities::createQueryFiltered($filters);
        $modulesConfig = InstanceProvider::getRepository()->getOmProdutoOrgVersao()->findVersionModulesConfiguration($conditions, $productVersionId);
        $modulesConfiguration = array();
        $productDevOpsId = null;
        if(!empty($modulesConfig)) {
            $modulesConfig = @json_decode($modulesConfig['MODULES_CONFIGURATION'], true);
            if(empty($modulesConfig)) {
                return $modulesConfiguration;
            }
            foreach($modulesConfig as $idx => $config) {
                if(!isset($config['onlyControl']) || !$config['onlyControl']) {
                    $product = $this->repositoryProduto->findOneBy(array('nrexternalProductId' => $config['devOpsId']));
                    $isProduct = $idx == self::PRODUCT_KEY;
                    if ($isProduct) {
                        $productDevOpsId = $config['devOpsId'];
                    }
                    $modulesConfiguration[] = array (
                        'ID'          => $config['devOpsId'],
                        'MAX_VERSION' => $isProduct ? null : $config['maxVersion'],
                        'PARENT'      => $isProduct ? null : $productDevOpsId,
                        'NAME'        => $product ? $product->getNmproduto() : ''
                    );
                }
            }
        }
        return $modulesConfiguration;
    }

    public function saveProductLineParams($id, $dsLinhaProduto, $idTpLinhaProduto, $idModoProc, $cdTpMedidaServ, $cdusuTpMedidaServ) {
        /** @var OmProduto $produto */
        $produto = InstanceProvider::getRepository()->getOmProduto()->find($id);
        $produto->setDslinhaproduto($dsLinhaProduto);
        $produto->setIdtplinhaproduto($idTpLinhaProduto);
        $produto->setIdmodoproc($idModoProc);
        $produto->setCdtpmedidaserv($cdTpMedidaServ);
        $produto->setCdusutpmedidaserv($cdusuTpMedidaServ);
        $this->entityManager->persist($produto);
        $this->entityManager->flush();
    }

    public function saveSystemProductLineParams($id, $cdSistema) {
        /** @var Entities\OmProdutoElemento $elemento */
        $elemento = InstanceProvider::getRepository()->getOmProdutoElemento()->findOneBy(array('id' => $id));
        $elemento->setCdsistema($cdSistema);
        $this->entityManager->persist($elemento);
        $this->entityManager->flush();
    }

    public function getSystemProductLine($filters, $idTpLinhaProduto) {
        $conditions = Utilities::createQueryFiltered($filters);
        return $this->repositoryProdutoElemento->getSystemProductLine($idTpLinhaProduto, $conditions);
    }

    /**
     * @param $cdtpmedidaserv
     * @param $filters
     * @return mixed
     */
    public function findCollectSettings($cdtpmedidaserv, $filters)
    {
        $conditions = Utilities::createQueryFiltered($filters);
        return InstanceProvider::getRepository()->getOmColetaMedidaVariavel()->findCollectSettings($cdtpmedidaserv, $conditions);
    }

    /**
     * @param $cdtpmedidaserv
     * @param $qtddiasaviso
     * @param $qtddiasreaplicar
     * @param $idativo
     * @param $phpscriptcount
     * @param $phpscriptlog
     * @return array
     * @throws \Exception
     */
    public function saveCollectSettings($id, $cdtpmedidaserv, $qtddiasaviso, $qtddiasreaplicar, 
                                        $idativo, $phpscriptcount, $phpscriptlog, $cdservorca, $idTpScript)
    {
        if($id){
            /** @var \Teknisa\Model\Teknisa\Entities\OmColetaMedidaVariavel $omColetaMedidaVariavel */
            $omColetaMedidaVariavel = InstanceProvider::getRepository()->getOmColetaMedidaVariavel()->find($id);
        } else {
            $omColetaMedidaVariavel = null;
        }

        /** @var \Teknisa\Model\Services\Entities\FctTipomedidaserv $fctTipomedidaserv */
        $omColetaMedidaVariavel = InstanceProvider::getFactory()->getColetaMedidaVariavel()->save($omColetaMedidaVariavel, $qtddiasaviso, $qtddiasreaplicar, 
            $idativo, $phpscriptcount, $phpscriptlog, $cdtpmedidaserv, $cdservorca, $idTpScript);
        $this->entityManager->persist($omColetaMedidaVariavel);
        $this->entityManager->flush();
        return array('ID' => $omColetaMedidaVariavel->getId());
    }

    public function deleteCollectSettings($id)
    {
        /** @var \Teknisa\Model\Teknisa\Entities\OmColetaMedidaVariavel $omColetaMedidaVariavel */
        $omColetaMedidaVariavel = InstanceProvider::getRepository()->getOmColetaMedidaVariavel()->find($id);
        $this->entityManager->remove($omColetaMedidaVariavel);
        $this->entityManager->flush();
    }

    public function findTiposervicoorca($conditions)
    {
        $filters = Utilities::createQueryFiltered($conditions);
        return InstanceProvider::getRepository()->getFctServicoOrca()->findTiposervicoorca($filters);
    }

    public function findConfigColeta($filters, $nrProdutoOrgId)
    {
        $conditions = Utilities::createQueryFiltered($filters);
        return InstanceProvider::getRepository()->getOmConfigColeta()->findConfigColeta($conditions, $nrProdutoOrgId);
    }

    /**
     * @param $id
     * @param $idAtivo
     * @param $qtdDiasReaplicar
     * @param $dtLimite
     * @param $phpScriptLog
     * @param $nrproductoId
     * @param $nrorgId
     * @return array
     * @throws \Exception
     */
    public function saveConfigColeta($id, $idAtivo, $qtdDiasReaplicar, $dtLimite, $phpScriptLog, $nrproductoId, $nrorgIds)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $omConfigColeta = null;
            $dateNow = new \DateTime();
            $currentUserId = $this->environment->getCurrentUserId();
            $currentOrganizationId = $this->environment->getCurrentOrganizationId();
            $dtLimite = \DateTime::createFromFormat('d/m/Y H:i:s', $dtLimite . ' 00:00:00');

            if($id) {
                /** @var Entities\OmConfigColeta $omConfigColeta */
                $omConfigColeta = InstanceProvider::getRepository()->getOmConfigColeta()->find($id);
            }
            $omConfigColeta = InstanceProvider::getFactory()->getConfigColeta()->save($omConfigColeta, $idAtivo, $qtdDiasReaplicar, $dtLimite, $phpScriptLog, $nrproductoId,
                                                                                        $currentUserId, $currentOrganizationId, $dateNow);
            $this->entityManager->persist($omConfigColeta);
            $this->entityManager->flush();

            $this->deleteOmConfigColetaOrg($omConfigColeta->getId());
            $this->entityManager->flush();

            foreach ($nrorgIds as $nrorgId) {
                $omConfigColetaOrg = InstanceProvider::getFactory()->getConfigColetaOrg()->create($omConfigColeta->getId(), $nrorgId, 'S', $currentUserId, $currentOrganizationId, $dateNow);
                $this->entityManager->persist($omConfigColetaOrg);
            }
            $this->entityManager->flush();
            $connection->commit();
            return array('ID' => $omConfigColeta->getId());
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function deleteOmConfigColetaOrg($nrconfcolId)
    {
        $omConfigColetaOrgs = InstanceProvider::getRepository()->getOmConfigColetaOrg()->findBy(array('nrconfcolId' => $nrconfcolId));
        foreach ($omConfigColetaOrgs as $omConfigColetaOrg) {
            $this->entityManager->remove($omConfigColetaOrg);
        }
    }

    public function deleteConfigColeta($id)
    {
        /** @var Entities\OmConfigColeta $produtoOrgVersao */
        $omConfigColeta = InstanceProvider::getRepository()->getOmConfigColeta()->find($id);
        $this->deleteOmConfigColetaOrg($omConfigColeta->getId());
        $this->entityManager->flush();
        $this->entityManager->remove($omConfigColeta);
        $this->entityManager->flush();
    }

    /**
     * @param OmProduto $produtoParaRemover
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    private function deleteProductRelations(OmProduto $produtoParaRemover)
    {
        $produtoElementos = $this->repositoryProdutoElemento->findBy(array(
            'nrprodutoId' => $produtoParaRemover->getId())
        );
        $produtoRepositorios = InstanceProvider::getRepository()->getOmProdutoRepositorio()->findBy(array(
            'nrprodutoId' => $produtoParaRemover->getId())
        );
        $produtoConfigMed = InstanceProvider::getRepository()->getOmConfigMedidaVariavel()->findBy(array(
            'nrprodutoId' => $produtoParaRemover->getId())
        );
        $produtoConfigColeta = InstanceProvider::getRepository()->getOmConfigColeta()->findBy(array(
            'nrprodutoId' => $produtoParaRemover->getId())
        );
        foreach ($produtoElementos as $produtoElemento) {
            $produtoOrgElementos = $this->repositoryProdutoOrgElemento->findBy(array(
                'nrprodutoElementoId' => $produtoElemento->getId())
            );
            foreach ($produtoOrgElementos as $produtoOrgElemento) {
                $this->entityManager->remove($produtoOrgElemento);
            }
            $this->entityManager->flush();
            $this->entityManager->remove($produtoElemento);
        }
        foreach ($produtoRepositorios as $produtoRepositorio) {
            $this->entityManager->flush();
            $this->entityManager->remove($produtoRepositorio);
        }
        foreach ($produtoConfigMed as $produtoConfig) {
            $this->entityManager->flush();
            $this->entityManager->remove($produtoConfig);
        }
        foreach ($produtoConfigColeta as $produtoConfigCol) {
            $produtoConfigColetaOrg = InstanceProvider::getRepository()->getOmConfigColetaOrg()->findBy(array(
                'nrconfcolId' => $produtoConfigCol->getId())
            );
            foreach ($produtoConfigColetaOrg as $produtoConfigColOrg) {
                $this->entityManager->remove($produtoConfigColOrg);
            }
            $this->entityManager->flush();
            $this->entityManager->remove($produtoConfigCol);
        }
    }
    
    public function validateVariableMeasureSettings($nrProdutoId, $organizationId) {
        $configMedidaVariavelRepository = InstanceProvider::getRepository()->getOmConfigMedidaVariavel();
        $productId  = InstanceProvider::getRepository()->getOmProduto()->find($nrProdutoId)->getNrexternalProductId();
        $produtoOrg = $this->repositoryProdutoOrg->findOneBy(array('nrprodutoId' => $nrProdutoId, 'nrorgId' => $organizationId));
        
        $variableMeasureConfig = $configMedidaVariavelRepository->findVariableMeasureByExternalProductId($productId);
        if(empty($variableMeasureConfig)) {
            return Messages::ALE_EMPTY_PRODUCT_VARIABEL_MEASURE_CONFIG;
        }
        
        $contractsCliente = $configMedidaVariavelRepository->findActualCustomerContracts(OmConfigMedidaVariavel::VARIABLE_MEASURE_COLLECT_CLIENT_TYPE, $productId, $organizationId);
        $contractsTeknisa = $configMedidaVariavelRepository->findActualCustomerContracts(OmConfigMedidaVariavel::VARIABLE_MEASURE_COLLECT_TEKNISA_TYPE, $productId, $organizationId);
        if(empty($contractsCliente) && empty($contractsTeknisa)) {
            return Messages::ALE_EMPTY_CONTRACTS_VARIABEL_MEASURE;
        }
        
        if($produtoOrg->getIdprodutosaas() == 'S') {
            $organizationId = Utilities::getTeknisaNrorgParameter();
        }
        $servers = $configMedidaVariavelRepository->findOrganizationProductionServers($productId, $organizationId);
        if(empty($servers) && !empty($contractsCliente)) {
            return Messages::ALE_EMPTY_PRODUCTION_SERVER_VARIABEL_MEASURE;
        }
        
        if($this->existsSQLVariableMeasure($variableMeasureConfig)) {
            $parameters = InstanceProvider::getRepository()->getOmProdutoColetor()->getProductCollector($nrProdutoId);
            $parameters = empty($parameters) ? null : $parameters[0];
            if(empty($parameters)) {
                $parameters = InstanceProvider::getRepository()->getOmParamOrg()->getOrganizationParameters($organizationId);
                $parameters = empty($parameters) ? null : $parameters[0];
                if(empty($parameters) || empty($parameters['IDCOLETOR']) || empty($parameters['DSSTRINGCONEXAO'])) {
                    $organization      = InstanceProvider::getRepository()->getOrganizacao()->findOneBy(array('nrorg' => $organizationId));
                    $conditions        = " AND OPD.IDAMBORG = '" . $organization->getIdorgsaas() . "' ";
                    $defaultParameters = InstanceProvider::getRepository()->getOmParamOrgDefault()->getDefaultParameters($conditions);
                    if(empty($defaultParameters)) {
                        return Messages::ALE_EMPTY_COLLECTOR_AND_DATABASE_VARIABEL_MEASURE;
                    }
                }
            }
        }
        
        return false;
    }
    
    private function existsSQLVariableMeasure($variableMeasureConfig) {
        $exists = false;
        foreach($variableMeasureConfig as $config) {
            if($config['IDTPSCRIPT'] == 'SQL') {
                $exists = true;
                break;
            }
        }
        return $exists;
    }

    public function getProductCollector($filters, $nrProdutoId)
    {
        $conditions = Utilities::createQueryFiltered($filters);
        return InstanceProvider::getRepository()->getOmProdutoColetor()->getProductCollector($nrProdutoId, $conditions);
    }

    public function saveProductCollector($id, $idColetor,  $dsStringConexao, $idUtWallet, $dsUsuarioBanco,
                                          $dsSenhaBanco, $idativo, $nrProdutoId, $idTpBanco) 
                                          {
        $dateNow      = new \DateTime();
        $currentOrgId = $this->environment->getCurrentOrganizationId();
        $codOperador  = $this->environment->getCurrentUserId();

        /** @var TeknisaEntities\OmProdutoColetor $productCollector */
        $productCollector = $id ? InstanceProvider::getRepository()->getOmProdutoColetor()->find($id) : null;
        if($productCollector) {
            if(!empty($dsSenhaBanco) && $productCollector->getDssenhabanco() != $dsSenhaBanco) {
                $dsSenhaBanco = OpensslCrypt::encrypt($dsSenhaBanco);
            }
        } else {
            $dsSenhaBanco = OpensslCrypt::encrypt($dsSenhaBanco);
        }
        $savedCollector = $this->factoryProduto->saveProductCollector($productCollector,
            $currentOrgId, $codOperador, $dateNow, $idColetor, $dsStringConexao, $idUtWallet,
            $dsUsuarioBanco, $dsSenhaBanco, $idativo, $nrProdutoId, $idTpBanco);
        $this->entityManager->persist($savedCollector);
        $this->entityManager->flush();
        return array(ltrim($savedCollector->getId(), '0'));
    }

    public function removeProductCollector($id) 
    {
        if($id) {
            $entity = InstanceProvider::getRepository()->getOmProdutoColetor()->find($id);
            $this->entityManager->remove($entity);
            $this->entityManager->flush();
        }
    }

    public function getProductOrgCollector($filters, $nrProdutoOrgId)
    {
        $mainProductOrg = null;
        $conditions = Utilities::createQueryFiltered($filters);
        $productOrg = $this->repositoryProdutoOrg->findOneBy(array('id' => $nrProdutoOrgId));
        $organization = InstanceProvider::getRepository()->getOrganizacao()->findOneBy(array('nrorg' => $productOrg->getNrorgId()));
        if($organization->getNrorgprincipal()) {
            $mainProductOrg = $this->repositoryProdutoOrg->findOneBy(array(
                'nrorgId' => $organization->getNrorgprincipal(),
                'nrprodutoId' => $productOrg->getNrprodutoId(),
            ));
            if($mainProductOrg) {
                $nrProdutoOrgId = $mainProductOrg->getId();
            }
        }
        $productOrgCollector = InstanceProvider::getRepository()->getOmProdutoOrgColetor()->getProductOrgCollector($nrProdutoOrgId, $conditions);
        
        if(!empty($productOrgCollector) && $mainProductOrg) {
            $productOrgCollector[0]['USING_MAIN_COLLECTOR'] = 'S';
        } else if(empty($productOrgCollector) && $mainProductOrg) {
            $productOrgCollector = array(array(
                'IDATIVO' => 'S',
                'USING_MAIN_COLLECTOR' => 'S',
            ));
        } else if(empty($productOrgCollector)) {
            $productOrgCollector = array(array(
                'IDATIVO' => 'S',
                'USING_MAIN_COLLECTOR' => 'N',
            ));
        }

        return $productOrgCollector;
    }

    public function saveProductOrgCollector($id, $idColetor,  $dsStringConexao, $idUtWallet, $dsUsuarioBanco,
                                            $dsSenhaBanco, $idativo, $nrProdutoOrgId, $idTpBanco) 
    {
        $dateNow      = new \DateTime();
        $currentOrgId = $this->environment->getCurrentOrganizationId();
        $codOperador  = $this->environment->getCurrentUserId();

        /** @var TeknisaEntities\OmProdutoOrgColetor $productOrgCollector */
        $productOrgCollector = $id ? InstanceProvider::getRepository()->getOmProdutoOrgColetor()->find($id) : null;
        if($productOrgCollector) {
            if(!empty($dsSenhaBanco) && $productOrgCollector->getDssenhabanco() != $dsSenhaBanco) {
                $dsSenhaBanco = OpensslCrypt::encrypt($dsSenhaBanco);
            }
        } else {
            $dsSenhaBanco = OpensslCrypt::encrypt($dsSenhaBanco);
        }
        $savedOrgCollector = $this->factoryProdutoOrg->saveProductOrgCollector($productOrgCollector,
            $currentOrgId, $codOperador, $dateNow, $idColetor, $dsStringConexao, $idUtWallet,
            $dsUsuarioBanco, $dsSenhaBanco, $idativo, $nrProdutoOrgId, $idTpBanco);
        $this->entityManager->persist($savedOrgCollector);
        $this->entityManager->flush();
        return array(ltrim($savedOrgCollector->getId(), '0'));
    }

    public function removeProductOrgCollector($id) 
    {
        if($id) {
            $entity = InstanceProvider::getRepository()->getOmProdutoOrgColetor()->find($id);
            $this->entityManager->remove($entity);
            $this->entityManager->flush();
        }
    }
}